/**
 * Copyright 2005-2015 Alfresco Software, Ltd. All rights reserved.
 * License rights for this program may be obtained from Alfresco Software, Ltd.
 * pursuant to a written agreement and any use of this program without such an
 * agreement is prohibited.
 */

package com.activiti;

import java.io.FileReader;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.util.Date;

import javax.sql.rowset.serial.SerialBlob;

import org.apache.commons.csv.CSVFormat;
import org.apache.commons.csv.CSVParser;
import org.apache.commons.csv.CSVRecord;

/**
 * @author jbarrez
 */
public class RestoreEvents {
	
	private static String jdbcUrl;
	private static String jdbcUsername;
	private static String jdbcPassword;
	private static String tablePrefix;
	private static String inputFile;
	
	public static void main(String[] args) throws Exception {
		
		System.out.println("Usage:");
		System.out.println("-DjdbcUrl=<jdbcUrl> -DjdbcUsername=<jdbcUsername> -DjdbcPassword<jdbcPassword> -DtablePrefix=<tablePrefix> -DbatchSize=<batchSize> -DinputFile=<inputFile>");
		System.out.println();

		jdbcUrl = System.getProperty("jdbcUrl");
		jdbcUsername = System.getProperty("jdbcUsername");
		jdbcPassword = System.getProperty("jdbcPassword");
		tablePrefix = System.getProperty("tablePrefix");
		inputFile = System.getProperty("inputFile");
		
		if (jdbcUrl == null) {
			throw new RuntimeException("Jdbc url is mandatory");
		}
		if (jdbcUsername == null) {
			throw new RuntimeException("Jdbc url is mandatory");
		}
		if (jdbcPassword == null) {
			throw new RuntimeException("Jdbc url is mandatory");
		}
		if (inputFile == null) {
			throw new RuntimeException("inputFile is mandatory");
		}

		printIfNotNull("Jdbc url", jdbcUrl);
		printIfNotNull("Jdbc username", jdbcUsername);
		printIfNotNull("Jdbc password", jdbcPassword);
		printIfNotNull("Table prefix", tablePrefix);
		printIfNotNull("input file", inputFile);
		
		long nrOfRowsProcessed = 0;
		
		Connection connection = DriverManager.getConnection(jdbcUrl, jdbcUsername, jdbcPassword);
		FileReader fileReader = new FileReader(inputFile);
		CSVParser csvFileParser = new CSVParser(fileReader, CSVFormat.DEFAULT.withHeader(Constants.HEADER));
		
		for (CSVRecord record : csvFileParser) {
			
			if (nrOfRowsProcessed > 0) {
			
				Long logNr = Long.valueOf(record.get(Constants.LOG_NR));
				String type = record.get(Constants.TYPE);
				String processDefinitionId = record.get(Constants.PROCESS_DEFINITION_ID);
				String processInstanceId = record.get(Constants.PROCESS_INSTANCE_ID);
				String executionId = record.get(Constants.EXECUTION_ID);
				String taskId = record.get(Constants.TASK_ID);
				Date timeStamp = Constants.DATE_FORMAT.parse(record.get(Constants.TIME_STAMP));
				String userId = record.get(Constants.USER_ID);
				String data = record.get(Constants.DATA);
				
				PreparedStatement stmt = connection.prepareStatement("insert into " + (tablePrefix != null ? ("." + tablePrefix) : "") + "ACT_EVT_LOG "
						+ "(LOG_NR_, TYPE_, PROC_DEF_ID_, PROC_INST_ID_, EXECUTION_ID_, TASK_ID_, TIME_STAMP_, USER_ID_, DATA_, IS_PROCESSED_) values (?, ?, ?, ?, ?, ?, ?, ?, ?, 0);");
				
				stmt.setLong(1, logNr);
				stmt.setString(2, type);
				stmt.setString(3, processDefinitionId);
				stmt.setString(4, processInstanceId);
				stmt.setString(5, executionId);
				stmt.setString(6, taskId);
				stmt.setDate(7, new java.sql.Date(timeStamp.getTime()));
				stmt.setString(8, userId);
				stmt.setBlob(9, new SerialBlob(data.getBytes("UTF-8")));
				
				stmt.execute();
				
			}
			
			nrOfRowsProcessed++;
			
			if (nrOfRowsProcessed % 1000 == 0) {
				System.out.println("Processed " + nrOfRowsProcessed + " rows");
			}
		}
		
		csvFileParser.close();

		System.out.println("All done");
	}
	
	private static void printIfNotNull(String helpText, Object o) {
		if (o != null) {
			System.out.println(helpText + " : " + o);
		}
	}
	

}
